//
// file: primitive3D_searchLightAltNoCorSoft.fx
// version: v1.5
// author: Ren712
//

//--------------------------------------------------------------------------------------
// Settings
//--------------------------------------------------------------------------------------
float3 sElementRotation = float3(0, 0, 0);
float3 sElementPosition = float3(0, 0, 0);
float4 sElementColor = float4(0, 0, 0, 0);
float sLightAttenuation = 1;
float sLightAttenuationPower = 2;
float sLightPhi = 0;
float sLightTheta = 0;
float sLightFalloff = 0;

float2 gDistFade = float2(250,150);

int2 sScrRes = float2(800,600);
bool fCullMode = 2;

float2 sHalfPixel = float2(0.00125,0.00166);

//--------------------------------------------------------------------------------------
// Variables set by MTA
//--------------------------------------------------------------------------------------
float4x4 gProjection : PROJECTION;
float4x4 gView : VIEW;
float4x4 gViewInverse : VIEWINVERSE;
float3 gCameraPosition : CAMERAPOSITION;
texture gDepthBuffer : DEPTHBUFFER;

static const float PI = 3.14159265f;
int CUSTOMFLAGS < string skipUnusedParameters = "yes"; >;

//--------------------------------------------------------------------------------------
// Sampler 
//--------------------------------------------------------------------------------------
sampler SamplerDepth = sampler_state
{
    Texture = (gDepthBuffer);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

//--------------------------------------------------------------------------------------
// Structures
//--------------------------------------------------------------------------------------
struct VSInput
{
    float3 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float4 Diffuse : COLOR0;
};

struct PSInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float DistFade : TEXCOORD1;
    float3 WorldPos : TEXCOORD2;
    float3 Normal : TEXCOORD3;
    float4 TexProj : TEXCOORD4;
    float4 Diffuse : COLOR0;
};

//--------------------------------------------------------------------------------------
// Create world matrix with world position and euler rotation
//--------------------------------------------------------------------------------------
float4x4 createWorldMatrix(float3 pos, float3 rot)
{
    float4x4 eleMatrix = {
        float4(cos(rot.z) * cos(rot.y) - sin(rot.z) * sin(rot.x) * sin(rot.y), 
                cos(rot.y) * sin(rot.z) + cos(rot.z) * sin(rot.x) * sin(rot.y), -cos(rot.x) * sin(rot.y), 0),
        float4(-cos(rot.x) * sin(rot.z), cos(rot.z) * cos(rot.x), sin(rot.x), 0),
        float4(cos(rot.z) * sin(rot.y) + cos(rot.y) * sin(rot.z) * sin(rot.x), sin(rot.z) * sin(rot.y) - 
                cos(rot.z) * cos(rot.y) * sin(rot.x), cos(rot.x) * cos(rot.y), 0),
        float4(pos.x,pos.y,pos.z, 1),
    };
    return eleMatrix;
}

//--------------------------------------------------------------------------------------
// Get value from the depth buffer
// Uses define set at compile time to handle RAWZ special case (which will use up a few more slots)
//--------------------------------------------------------------------------------------
float FetchDepthBufferValue( float2 uv )
{
    float4 texel = tex2D(SamplerDepth, uv);
#if IS_DEPTHBUFFER_RAWZ
    float3 rawval = floor(255.0 * texel.arg + 0.5);
    float3 valueScaler = float3(0.996093809371817670572857294849, 0.0038909914428586627756752238080039, 1.5199185323666651467481343000015e-5);
    return dot(rawval, valueScaler / 255.0);
#else
    return texel.r;
#endif
}
 
//--------------------------------------------------------------------------------------
// Use the last scene projecion matrix to linearize the depth value a bit more
//--------------------------------------------------------------------------------------
float Linearize(float posZ)
{
    return gProjection[3][2] / (posZ - gProjection[2][2]);
}

//--------------------------------------------------------------------------------------
// Get soft particle spread 
//--------------------------------------------------------------------------------------
float countDepthSpread(float sceneDepth, float pixelDepth, float depthSpread, float attenuationPower)
{
    if (pixelDepth > sceneDepth) 
        return pow(saturate((sceneDepth - (pixelDepth - depthSpread)) / depthSpread), attenuationPower); 
    else
        return 1;
}

//--------------------------------------------------------------------------------------
// Vertex Shader 
//--------------------------------------------------------------------------------------
PSInput VertexShaderFunction(VSInput VS)
{
    PSInput PS = (PSInput)0;
	
    // create WorldMatrix for the quad
    float4x4 sWorld = createWorldMatrix(sElementPosition, sElementRotation);
	
    // set proper position and scale of the quad
    VS.Position.xyz = float3(- 0.5 + VS.TexCoord.xy, 0);
	
    // shape the cylinder
    VS.Position.z = cos(2 * VS.Position.x * PI) / 2;
	VS.Position.x = sin(2 * VS.Position.x * PI) / 2;
    VS.Position.y += 0.5;
	
    // shape the cone
    VS.Position.xz *= 2 * tan(sLightPhi);
    VS.Position.xz *= VS.Position.y;
	
    // create normal vector based on cone shape
    float3 posVec = normalize(VS.Position.xyz);
    float3 newPos = (cos(sLightPhi / 2) * VS.Position.y) * posVec;
	float3 oldPos = float3(0, VS.Position.y, 0);
    float3 normal = normalize(oldPos - newPos);
    PS.Normal = mul(normal, (float3x3)sWorld).xyz;
	
    // scale the position
    VS.Position.xyz *= sLightAttenuation;

    // flip texCoords.x
    VS.TexCoord.x = 1 - VS.TexCoord.x;
	
    // calculate screen position of the vertex
    float4 wPos = mul(float4( VS.Position, 1), sWorld);
    PS.WorldPos = wPos.xyz;
    float4 vPos = mul(wPos, gView);
    PS.TexProj.w = vPos.z / vPos.w;	
    PS.Position = mul(vPos, gProjection);
	
    // set texCoords for projective texture
    float projectedX = (0.5 * (PS.Position.w + PS.Position.x));
    float projectedY = (0.5 * (PS.Position.w - PS.Position.y));
    PS.TexProj.xyz = float3(projectedX, projectedY, PS.Position.w);  

    // get clip values
    float nearClip = - gProjection[3][2] / gProjection[2][2];
    float farClip = (gProjection[3][2] / (1 - gProjection[2][2]));	
	
    // fade object
    float DistFromCam = distance(gViewInverse[3].xyz, sElementPosition);
    float2 DistFade = float2(min(gDistFade.x, farClip + sLightAttenuation / 2), min(gDistFade.y, farClip + sLightAttenuation /2 - (gDistFade.x - gDistFade.y)));
    PS.DistFade = saturate((DistFromCam - DistFade.x)/(DistFade.y - DistFade.x));

    // pass texCoords and vertex color to PS
    PS.TexCoord = VS.TexCoord;
    PS.Diffuse = VS.Diffuse * sElementColor;
	
    return PS;
}

PSInput VertexShaderFunctionNoDB(VSInput VS)
{
    PSInput PS = (PSInput)0;
	
    // create WorldMatrix for the quad
    float4x4 sWorld = createWorldMatrix(sElementPosition, sElementRotation);
	
    // set proper position and scale of the quad
    VS.Position.xyz = float3(- 0.5 + VS.TexCoord.xy, 0);
	
    // shape the cylinder
    VS.Position.z = cos(2 * VS.Position.x * PI) / 2;
	VS.Position.x = sin(2 * VS.Position.x * PI) / 2;
    VS.Position.y += 0.5;
	
    // shape the cone
    VS.Position.xz *= 2 * tan(sLightPhi);
    VS.Position.xz *= VS.Position.y;
	
    // create normal vector based on cone shape
    float3 posVec = normalize(VS.Position.xyz);
    float3 newPos = (cos(sLightPhi / 2) * VS.Position.y) * posVec;
	float3 oldPos = float3(0, VS.Position.y, 0);
    float3 normal = normalize(oldPos - newPos);
    PS.Normal = mul(normal, (float3x3)sWorld).xyz;
	
    // scale the position
    VS.Position.xyz *= sLightAttenuation;

    // flip texCoords.x
    VS.TexCoord.x = 1 - VS.TexCoord.x;
	
    // calculate screen position of the vertex
    float4 wPos = mul(float4( VS.Position, 1), sWorld);
    PS.WorldPos = wPos.xyz;
    float4 vPos = mul(wPos, gView);	
    PS.Position = mul(vPos, gProjection);

    // get clip values
    float nearClip = - gProjection[3][2] / gProjection[2][2];
    float farClip = (gProjection[3][2] / (1 - gProjection[2][2]));	
	
    // fade object
    float DistFromCam = distance(gViewInverse[3].xyz, sElementPosition);
    float2 DistFade = float2(min(gDistFade.x, farClip + sLightAttenuation / 2), min(gDistFade.y, farClip + sLightAttenuation /2 - (gDistFade.x - gDistFade.y)));
    PS.DistFade = saturate((DistFromCam - DistFade.x)/(DistFade.y - DistFade.x));

    // pass texCoords and vertex color to PS
    PS.TexCoord = VS.TexCoord;
    PS.Diffuse = VS.Diffuse * sElementColor;
	
    return PS;
}

//--------------------------------------------------------------------------------------
// Pixel shaders 
//--------------------------------------------------------------------------------------
float4 PixelShaderFunction(PSInput PS) : COLOR0
{
    // get projective texture coords	
    float2 TexProj = PS.TexProj.xy / PS.TexProj.z;
    TexProj += sHalfPixel.xy;

    // compute the distance attenuation factor
    float fDistance = distance(sElementPosition, PS.WorldPos);

    // compute the attenuation
    float fAttenuation = 1 - saturate(fDistance / sLightAttenuation);
    fAttenuation = pow(fAttenuation, sLightAttenuationPower);
	
    // apply diffuse color
    float4 finalColor = PS.Diffuse;

    // add a light specular
    float NdotL = dot(normalize(PS.Normal), normalize(gCameraPosition - PS.WorldPos));
    NdotL = pow(abs(NdotL), 2);
    fAttenuation *= saturate(NdotL);
	
    // apply attenuation
    finalColor.rgb *= saturate(fAttenuation);
	
    // apply distance fade
    finalColor.a *= saturate(PS.DistFade);
	
    // calculate soft particle effect
    float depth = Linearize(FetchDepthBufferValue(TexProj.xy));
    float depthMul = countDepthSpread(depth, PS.TexProj.w, PS.TexCoord.y * sLightAttenuation * 0.1, 1.5);
    finalColor.a *= depthMul;

    return saturate(finalColor);
}

float4 PixelShaderFunctionNoDB(PSInput PS) : COLOR0
{
    // compute the distance attenuation factor
    float fDistance = distance(sElementPosition, PS.WorldPos);

    // compute the attenuation
    float fAttenuation = 1 - saturate(fDistance / sLightAttenuation);
    fAttenuation = pow(fAttenuation, sLightAttenuationPower);
	
    // apply diffuse color
    float4 finalColor = PS.Diffuse;

    // add a light specular
    float NdotL = dot(normalize(PS.Normal), normalize(gCameraPosition - PS.WorldPos));
    NdotL = pow(abs(NdotL), 2);
    fAttenuation *= saturate(NdotL);
	
    // apply attenuation
    finalColor.rgb *= saturate(fAttenuation);
	
    // apply distance fade
    finalColor.a *= saturate(PS.DistFade);

    return saturate(finalColor);
}

//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique dxDrawImage4D_searchLightAltNoCorSoft
{
  pass P0
  {
    ZEnable = false;
    CullMode = fCullMode;
    ShadeMode = Gouraud;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = One;
    AlphaTestEnable = true;
    AlphaRef = 1;
    AlphaFunc = GreaterEqual;
    Lighting = false;
    FogEnable = false;
    VertexShader = compile vs_2_0 VertexShaderFunction();
    PixelShader  = compile ps_2_0 PixelShaderFunction();
  }
}

technique dxDrawPrimitive3Dfx_searchLightAltNoCorSoft_NoDB
{
  pass P0
  {
    ZEnable = true;
    ZFunc = LessEqual;
    ZWriteEnable = false;
    CullMode = fCullMode;
    ShadeMode = Gouraud;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = One;
    AlphaTestEnable = true;
    AlphaRef = 1;
    AlphaFunc = GreaterEqual;
    Lighting = false;
    FogEnable = false;
    VertexShader = compile vs_2_0 VertexShaderFunctionNoDB();
    PixelShader  = compile ps_2_0 PixelShaderFunctionNoDB();
  }
}

// Fallback
technique fallback
{
  pass P0
  {
    // Just draw normally
  }
}
